
# script based on fwscripts : http://www.malibyte.net/iptables/scripts/fwscripts.html
# Thanks to Jeff Carlson (jeff@ultimateevil.org) for his assistance re: DHCP and several other issues,
# Rohan Amin (rohan@rohanamin.com) and Erik Wasser (erik.wasser@iquer.com) for help with the port-forwarding
# routine, and Nate Waddoups for his quick PPTP hack.
#
# Modified by Raum 01/06/05 (raum@forward.to, http://linbox.free.fr)
#    => SSH chain to interrupt SSH scanning
#    => TOS et SYSCTL from HomeLANSecurity (www.unixpage.com)

# chkconfig: 345 11 91
#
# description: IP Firewall startup/shutdown script for $IPTABLES
#
# probe: true
#

#
# CONSTANTS - Do not edit
#

ANYWHERE="0.0.0.0/0"                    # Match any IP address
BROADCAST_SRC="0.0.0.0"                 # Broadcast Source Address
BROADCAST_DEST="255.255.255.255"        # Broadcast Destination Address
CLASS_A="10.0.0.0/8"                    # Class-A Private (RFC-1918) Networks
CLASS_B="172.16.0.0/12"                 # Class-B Private (RFC-1918) Networks
CLASS_C="192.168.0.0/16"                # Class-C Private (RFC-1918) Networks
CLASS_D_MULTICAST="224.0.0.0/4"         # Class-D Multicast Addresses
CLASS_E_RESERVED_NET="240.0.0.0/5"      # Class-E Reserved Addresses
PRIVPORTS="0:1023"                      # Well-Known, Privileged Port Range
UNPRIVPORTS="1024:65535"                # Unprivileged Port Range
TRACEROUTE_SRC_PORTS="32769:65535"      # Traceroute Source Ports
TRACEROUTE_DEST_PORTS="33434:33523"     # Traceroute Destination Ports


#
# The Loopback interface defines should not be
# edited unless your Linux distribution defines
# these differently.
#

LOOPBACK_INTERFACE="lo"                 # The loopback interface
LOOPBACK_NETWORK="127.0.0.0/8"          # Reserved Loopback Address Range
IPTABLES="/sbin/iptables"
FW_CONF="/etc/firewall/firewall.conf"
LOGFILE="/etc/firewall/last.log"

#
# Source function library.
#

. /etc/rc.d/init.d/functions

IPSYSCTL() {
   # source from homeLANsecurity (www.unixpages.com)

   # Enables kernel timestamps as defined in RFC 1323 used to calculate the
   # Round Trip Measurement in a better way than the retransmission timeout method
   if [ -f /proc/sys/net/ipv4/tcp_timestamps ]; then
      echo 1 > /proc/sys/net/ipv4/tcp_timestamps
   fi

   # Disable logging of packets from illegal addresses
   if [ -f /proc/sys/net/ipv4/conf/all/log_martians ]; then
      echo 0 > /proc/sys/net/ipv4/conf/all/log_martians
   fi

   # Disables logging of RFC 1122 violations (bogus responses to broadcast frames)
   if [ -f /proc/sys/net/ipv4/icmp_ignore_bogus_error_responses ]; then
      echo 1 > /proc/sys/net/ipv4/icmp_ignore_bogus_error_responses
   fi

   # Disable TCP Explicit Congestion Notification Support
   if [ -f /proc/sys/net/ipv4/tcp_ecn ]; then
      echo 0 > /proc/sys/net/ipv4/tcp_ecn
   fi

   # Proxy ARP is a system of automatically answering ARP queries for other hosts
   # (1=ON 2=OFF)
   if [ -f /proc/sys/net/ipv4/conf/$EXTERNAL_INTERFACE/proxy_arp ]; then
      echo 2 > /proc/sys/net/ipv4/conf/$EXTERNAL_INTERFACE/proxy_arp
   fi

   # Enable proxy ARP for LANIFACE   (1=ON 2=OFF)
   if [ -f /proc/sys/net/ipv4/conf/$INTERNAL_INTERFACE/proxy_arp ]; then
      echo 1 > /proc/sys/net/ipv4/conf/$INTERNAL_INTERFACE/proxy_arp
   fi

   # Enable flood protection 
   if [ -f /proc/sys/net/ipv4/tcp_syncookies ]; then
      echo 1 > /proc/sys/net/ipv4/tcp_syncookies
   fi

   # Enable source address verification to prevent spoofing
   if [ -f /proc/sys/net/ipv4/conf/all/rp_filter ]; then
      for x in /proc/sys/net/ipv4/conf/*/rp_filter; do
      echo 1 > $x; done
   fi

   # Disable source routed packets to help prevent outside access to LAN
   if [ -f /proc/sys/net/ipv4/conf/all/accept_source_route ]; then
      for x in /proc/sys/net/ipv4/conf/*/accept_source_route; do
      echo 0 > $x; done 
   fi

   # Accept redirects only from gateways in the default gateways list
   if [ -f /proc/sys/net/ipv4/conf/all/secure_redirects ]; then
      echo 1 > /proc/sys/net/ipv4/conf/all/secure_redirects
   fi

   # Disable ICMP redirects
   if [ -f /proc/sys/net/ipv4/conf/all/send_redirects ]; then
      echo 0 > /proc/sys/net/ipv4/conf/all/send_redirects
   fi

   # Disable acceptance of ICMP redirects to avoid malicious routing changes
   if [ -f /proc/sys/net/ipv4/conf/all/accept_redirects ]; then
      echo 0 > /proc/sys/net/ipv4/conf/all/accept_redirects
   fi

   # Ignore broadcast ICMP echo requests to prevent
   # becoming a Smurf attack amplifier
   if [ -f /proc/sys/net/ipv4/icmp_echo_ignore_broadcasts ]; then
      echo 1 > /proc/sys/net/ipv4/icmp_echo_ignore_broadcasts
   fi

   # Enables the Selective Acknowledgements algoorithm as defined in RFC 2883
   # which helps to shore up poor internet connections
   if [ -f /proc/sys/net/ipv4/tcp_sack ]; then
      echo 1 > /proc/sys/net/ipv4/tcp_sack
   fi

   # Enables the Forward Acknowledgement algorithm
   # geared at congestion control 
   # This option is dependent upon tcp_sack
   if [ -f /proc/sys/net/ipv4/tcp_fack ]; then
      echo 1 > /proc/sys/net/ipv4/tcp_fack
   fi

   # Required to send duplicate SACKs as defined in RFC 2883
   # This option is used to tell the sender that
   # duplicate packets have been received
   if [ -f /proc/sys/net/ipv4/tcp_dsack ]; then
      echo 1 > /proc/sys/net/ipv4/tcp_dsack
   fi

   # Enables window scaling as it is defined in RFC 1323 allowing windows to be a
   # larger size than 65kb, helping to reduce bandwidth loss in fast connections
   if [ -f /proc/sys/net/ipv4/tcp_window_scaling ]; then
      echo 1 > /proc/sys/net/ipv4/tcp_window_scaling
   fi

   # Controls how much a TCP packet may be reordered in a stream without
   # assuming that the packet was lost somewhere on the way
   if [ -f /proc/sys/net/ipv4/tcp_reordering ]; then
      echo 5 > /proc/sys/net/ipv4/tcp_reordering            # Default=3
   fi

   # Adjust connection tracking timeout value
   # Default=600 (600 seconds or 10 minutes)
   if [ -f /proc/sys/net/ipv4/netfilter/ip_ct_generic_timeout ]; then
      echo 120 > /proc/sys/net/ipv4/netfilter/ip_ct_generic_timeout 
   fi

   # Increase maximum limit of connections to track
   if [ -f /proc/sys/net/ipv4/ip_conntrack_max ]; then
      echo 8192 > /proc/sys/net/ipv4/ip_conntrack_max       # Default=2048
   fi

   # The maximum length of the queue (i.e. the number packets delivered to
   # userspace with no verdict issued back)
   if [ -f /proc/sys/net/ipv4/ip_queue_maxlen ]; then
      echo 2048 > /proc/sys/net/ipv4/ip_queue_maxlen        # Default=1024
   fi

   # Maximum number of packets queued in INPUT (used when the interface receives
   # packets faster than it can process them)
   if [ -f /proc/sys/net/core/netdev_max_backlog ]; then
      echo 4096 > /proc/sys/net/core/netdev_max_backlog
   fi

   # Adjusts the write (transmit) memory buffer
   echo 262144 > /proc/sys/net/core/wmem_default            # Default=131072
   echo 262144 > /proc/sys/net/core/wmem_max                # Default=131072
   echo '8196 32768 262144' > /proc/sys/net/ipv4/tcp_wmem   # Default=4096 16384 131072

   # Adjusts the receive memory buffer
   echo 174760 > /proc/sys/net/core/rmem_default            # Default=87380
   echo 349520 > /proc/sys/net/core/rmem_max                # Default=174760
   echo '8196 174760 349520' > /proc/sys/net/ipv4/tcp_rmem  # Default=4096 87380 174760

   # Seconds before a connection to timeout
   if [ -f /proc/sys/net/ipv4/tcp_fin_timeout ]; then
      echo 30 > /proc/sys/net/ipv4/tcp_fin_timeout          # Default=180
   fi

   # Seconds before a stall connection is killed
   if [ -f /proc/sys/net/ipv4/tcp_keepalive_time ]; then
      echo 1800 > /proc/sys/net/ipv4/tcp_keepalive_time     # Default=7200
   fi
   if [ $VERBOSE -gt 0 ]; then
	   echo "firewall: SYSCTL applied"
   fi

}

TOS() {
# source from homeLANsecurity (www.unixpages.com)

#  0x00 = Normal-Service
#  0x02 = Minimize-Cost
#  0x04 = Maximize-Reliability
#  0x08 = Maximize-Throughput
#  0x10 = Minimize-Delay

# Values were partially determined by RFC 1060/1349
   $IPTABLES -t mangle -A PREROUTING -p tcp \
      -m multiport --dports 443 -j TOS --set-tos 0x04
   $IPTABLES -t mangle -A PREROUTING -p tcp \
      -m multiport --dports 20,80,4662 -j TOS --set-tos 0x08
   $IPTABLES -t mangle -A PREROUTING -p tcp \
      -m multiport --dports 21,22,25,53 -j TOS --set-tos 0x10
   $IPTABLES -t mangle -A PREROUTING -p udp \
      -m multiport --dports 53 -j TOS --set-tos 0x10
   $IPTABLES -t mangle -A PREROUTING -p icmp --j TOS --set-tos 0x10

   if [ $VERBOSE -gt 0 ]; then
	   echo "firewall: TOS applied"
   fi

}

#
# See how we were called.
#

case "$1" in
  start)
        echo "Starting Firewall services"
        echo "firewall: Configuring Firewall Rules using $IPTABLES"

        # Remove any existing rules from all chains
        $IPTABLES -F
        $IPTABLES -F -t nat
        $IPTABLES -F -t mangle

        # Set the default policy to drop
        $IPTABLES -P INPUT   DROP
        $IPTABLES -P OUTPUT  DROP
        $IPTABLES -P FORWARD DROP

        # Allow unlimited traffic on the loopback interface
        $IPTABLES -A INPUT -i $LOOPBACK_INTERFACE -j ACCEPT
        $IPTABLES -A OUTPUT -o $LOOPBACK_INTERFACE -j ACCEPT

        # A bug that showed up as of the Red Hat 7.2 release results in
        # the following 5 default policies breaking the firewall
        # initialization:

        #     fgrep -q '7.2' /etc/redhat-release
        #     if [ $? -ne 0 ] ; then
        #       $IPTABLES -t nat    -P PREROUTING  DROP
        #       $IPTABLES -t nat    -P OUTPUT      DROP
        #       $IPTABLES -t nat    -P POSTROUTING DROP

        #       $IPTABLES -t mangle -P PREROUTING  DROP
        #       $IPTABLES -t mangle -P OUTPUT      DROP
        #     fi

        # Remove any pre-existing user-defined chains
        $IPTABLES -X
        $IPTABLES -X -t nat
        $IPTABLES -X -t mangle

        # Zero counts
        $IPTABLES -Z

        # Open the configuration file
        if [ -f $FW_CONF ]; then
            . $FW_CONF
        else
            # Turn off IP Forwarding & Masquerading
            echo 0 >/proc/sys/net/ipv4/ip_forward

            # Turn off dynamic IP hacking
            echo "0" > /proc/sys/net/ipv4/ip_dynaddr

            # Allow unlimited traffic on the loopback interface
            $IPTABLES -A INPUT -i $LOOPBACK_INTERFACE -j ACCEPT
            $IPTABLES -A OUTPUT -o $LOOPBACK_INTERFACE -j ACCEPT

            if [ $MASQUERADING -gt 0 ]; then
               # Allow unlimited local traffic on the internal interface
               $IPTABLES -A INPUT -i $INTERNAL_INTERFACE -j ACCEPT
               $IPTABLES -A OUTPUT -o $INTERNAL_INTERFACE -j ACCEPT
            fi

            echo "firewall: No configuration file found at /etc/firewall/firewall.conf; "
            echo "firewall: default policies set to DROP on INPUT/OUTPUT/FORWARD chains."
            exit 1
        fi


        #$IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -p udp --dport 12857 -j DROP
        #$IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -p tcp --dport 12857 -j DROP
        $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -p tcp --dport 4000 -j DROP
        $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -p tcp --dport 4001 -j DROP
        $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -p tcp --dport 4080 -j DROP
        $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -p tcp --dport 1213 -j DROP


        #iptables -A INPUT -p tcp --dport 8080 -d 82.67.38.46/32 -j DROP

        #
        # If your IP address is dynamically assigned by a DHCP server,
        # your DHCP server's IP address and this machine's IP address are
        # obtained from /etc/dhcpc/hostinfo-$EXTERNAL_INTERFACE or
        # /etc/dhcpc/dhcpcd-$EXTERNAL_INTERFACE.info.
        #


        if [ $DHCP -gt 0 ]; then

          # Grab external IP address if already assigned

          EXTERNAL_IP=$( ifconfig $EXTERNAL_INTERFACE | grep 'inet[^6]' | sed 's/[a-zA-Z:]//g' | awk '{print $1}' )
          if [ -n $EXTERNAL_IP ]; then
            EXT_NETMASK=$( ifconfig $EXTERNAL_INTERFACE | grep 'inet[^6]' | sed 's/[a-zA-Z:]//g' | awk '{print $3}' )
            EXTERNAL_NETWORK=$( ipcalc -n $EXTERNAL_IP $EXT_NETMASK | cut -d\= -f2 )
            BROADCAST_NET=$( ipcalc -b $EXTERNAL_IP $EXT_NETMASK | cut -d\= -f2 )
          fi

          # Turn on dynamic IP hacking
          echo "1" > /proc/sys/net/ipv4/ip_dynaddr


          # Incoming DHCPOFFER from available DHCP servers
          $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -p udp \
                   -s 0.0.0.0         --sport 67 \
                   -d 255.255.255.255 --dport 68 -j ACCEPT

          # Initialization of rebinding: No lease or Lease time expired.
          $IPTABLES -A OUTPUT -o $EXTERNAL_INTERFACE -p udp   \
                   -s 0.0.0.0         --sport 68 \
                   -d 255.255.255.255 --dport 67 -j ACCEPT

          # Fall back to initialization
          # The client knows its server, but has either lost its
          # lease, or else needs to reconfirm the IP address after
          # rebooting.
          $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -p udp \
                   -s $DHCP_SERVER --sport 67     \
                   -d 255.255.255.255 --dport 68 -j ACCEPT
          $IPTABLES -A OUTPUT -o $EXTERNAL_INTERFACE -p udp \
                   -s 255.255.255.255 --sport 68 \
                   -d $DHCP_SERVER --dport 67 -j ACCEPT

          # As a result of the above, we're supposed to change our IP
          # address with this message, which is addressed to our new
          # address before the dhcp client has received the update.
          # Depending on the server implementation, the destination
          # address can be the new IP address, the subnet address, or
          # the limited broadcast address.

          # If the network subnet address is used as the destination,
          # the next rule must allow incoming packets destined to the
          # subnet address, and the rule must preceed any general rules
          # that block such incoming broadcast packets.

          $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -p udp \
                   -s $DHCP_SERVER --sport 67     \
                   --dport 68 -j ACCEPT

          # Lease renewal
          $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -p udp \
                   -s $DHCP_SERVER --sport 67     \
                   -d $EXTERNAL_IP --dport 68 -j ACCEPT
          $IPTABLES -A OUTPUT -o $EXTERNAL_INTERFACE -p udp \
                   -s $EXTERNAL_IP --sport 68       \
                   -d $DHCP_SERVER --dport 67 -j ACCEPT

          echo "firewall: DHCP Client configured"

          else
            # External IP assigned without DHCP (i.e. static); get some more info
            EXT_NETMASK=$( ifconfig $EXTERNAL_INTERFACE | grep 'inet[^6]' | sed 's/[a-zA-Z:]//g' | awk '{print $3}' )
            EXTERNAL_NETWORK=$( ipcalc -n $EXTERNAL_IP $EXT_NETMASK | cut -d\= -f2 )
            BROADCAST_NET=$( ipcalc -b $EXTERNAL_IP $EXT_NETMASK | cut -d\= -f2 )

        fi


        #
        # Refuse directed broadcasts; you may choose not to log these, as they can fill up your logs quickly
        #

#       $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -d $EXTERNAL_NETWORK \
#             -m limit --limit 1/s            \
#             -j LOG --log-prefix "[Directed Broadcast] "
        $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -d $EXTERNAL_NETWORK -j DROP
#       $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -d $BROADCAST_NET \
#             -m limit --limit 1/s            \
#             -j LOG --log-prefix "[Directed Broadcast] "
        $IPTABLES -A INPUT  -i $EXTERNAL_INTERFACE -d $BROADCAST_NET -j DROP

        # Refuse limited broadcasts
#       $IPTABLES -A INPUT  -i $EXTERNAL_INTERFACE -d 255.255.255.255 \
#             -m limit --limit 1/s                    \
#             -j LOG --log-prefix "[Limited Broadcast] "
        $IPTABLES -A INPUT  -i $EXTERNAL_INTERFACE -d 255.255.255.255 -j DROP



        #
        # Edit these to match the number of servers or connections
        # you support.
        #

        # X Window port allocation begins at 6000 and increments
        # for each additional server running from 6000 to 6063.

        XWINDOW_PORTS="6000:6063"               # (TCP) X Windows

        # SSH starts at 1023 and works down to 513 for each additional
        # simultaneous incoming connection.

       # SSH chain, to limit amount of connection per minute and drop SSH scan
       $IPTABLES -N SSHnD                 # Define custom DROP chain

       $IPTABLES -A SSHnD -p tcp -m limit --limit 1/m -j ACCEPT
       $IPTABLES -A SSHnD -j LOG --log-prefix "[SSH drop] " --log-level=info
       $IPTABLES -A SSHnD -j DROP

        #
        # Iptables allows creation of customized chains.  The -l (log) flag no longer
        # exists.  This is a custom chain which allows logging of DROPped packets.
        #

        $IPTABLES -N LnD                 # Define custom DROP chain

        $IPTABLES -A LnD -p tcp -m limit --limit 1/s -j LOG --log-prefix "[TCP drop] " --log-level=info
        $IPTABLES -A LnD -p udp -m limit --limit 1/s -j LOG --log-prefix "[UDP drop] " --log-level=info
        $IPTABLES -A LnD -p icmp -m limit --limit 1/s -j LOG --log-prefix "[ICMP drop] " --log-level=info
        $IPTABLES -A LnD -f -m limit --limit 1/s -j LOG --log-prefix "[FRAG drop] " --log-level=info
        $IPTABLES -A LnD -j DROP

        #
        # This custom chain logs, then REJECTs packets.
        #

        $IPTABLES -N LnR                 # Define custom REJECT chain

        $IPTABLES -A LnR -p tcp -m limit --limit 1/s -j LOG --log-prefix "[TCP reject] " --log-level=info
        $IPTABLES -A LnR -p udp -m limit --limit 1/s -j LOG --log-prefix "[UDP reject] " --log-level=info
        $IPTABLES -A LnR -p icmp -m limit --limit 1/s -j LOG --log-prefix "[ICMP reject] " --log-level=info
        $IPTABLES -A LnR -f -m limit --limit 1/s -j LOG --log-prefix "[FRAG reject] " --log-level=info
        $IPTABLES -A LnR -j REJECT

        #
        # This chain logs, then DROPs "Xmas" and Null packets which might indicate a port-scan attempt
        #

        $IPTABLES -N ScanD               # Define custom chain for possible port-scans

        $IPTABLES -A ScanD -p tcp -m limit --limit 1/s -j LOG --log-prefix "[TCP Scan?] "
        $IPTABLES -A ScanD -p udp -m limit --limit 1/s -j LOG --log-prefix "[UDP Scan?] "
        $IPTABLES -A ScanD -p icmp -m limit --limit 1/s -j LOG --log-prefix "[ICMP Scan?] "
        $IPTABLES -A ScanD -f -m limit --limit 1/s -j LOG --log-prefix "[FRAG Scan?] "
        $IPTABLES -A ScanD -j DROP


        #
        # This chain limits the number of new incoming connections to preventing DDoS attacks
        #

        $IPTABLES -N DDoS                # Define custom chain for possible DDoS attacks

        $IPTABLES -A DDoS -m limit --limit 12/s --limit-burst 24 -j RETURN
        $IPTABLES -A DDoS -j LOG --log-prefix "[DDos Attack?] "
        $IPTABLES -A DDoS -j DROP


        #
        # This chain drops connections from IANA reserved IP blocks
        #

        $IPTABLES -N IANA

        $IPTABLES -A IANA -p tcp -m limit --limit 1/s -j LOG --log-prefix "[IANA Reserved - TCP] " --log-level=info
        $IPTABLES -A IANA -p udp -m limit --limit 1/s -j LOG --log-prefix "[IANA Reserved - UDP] " --log-level=info
        $IPTABLES -A IANA -p icmp -m limit --limit 1/s -j LOG --log-prefix "[IANA Reserved - ICMP] " --log-level=info
        $IPTABLES -A IANA -f -m limit --limit 1/s -j LOG --log-prefix "[IANA Reserved - FRAG] " --log-level=info
        $IPTABLES -A IANA -j DROP


        #
        # This chain drops connections from IPs in the firewall.banned file
        #

        $IPTABLES -N Banned

        $IPTABLES -A Banned -p tcp -m limit --limit 1/s -j LOG --log-prefix "[TCP Banned] " --log-level=info
        $IPTABLES -A Banned -p udp -m limit --limit 1/s -j LOG --log-prefix "[UDP Banned] " --log-level=info
        $IPTABLES -A Banned -p icmp -m limit --limit 1/s -j LOG --log-prefix "[ICMP Banned] " --log-level=info
        $IPTABLES -A Banned -f -m limit --limit 1/s -j LOG --log-prefix "[FRAG Banned] " --log-level=info
        $IPTABLES -A Banned -j DROP


        #
        # Disallow packets frequently used by port-scanners
        #

        # All of the bits are cleared
        $IPTABLES -A INPUT -p tcp --tcp-flags ALL NONE -j ScanD

        # SYN and FIN are both set
        $IPTABLES -A INPUT -p tcp --tcp-flags SYN,FIN SYN,FIN -j ScanD

        # SYN and RST are both set
        $IPTABLES -A INPUT -p tcp --tcp-flags SYN,RST SYN,RST -j ScanD

        # FIN and RST are both set
        $IPTABLES -A INPUT -p tcp --tcp-flags FIN,RST FIN,RST -j ScanD

        # FIN is the only bit set, without the expected accompanying ACK
        $IPTABLES -A INPUT -p tcp --tcp-flags ACK,FIN FIN -j ScanD

        # PSH is the only bit set, without the expected accompanying ACK
        $IPTABLES -A INPUT -p tcp --tcp-flags ACK,PSH PSH -j ScanD

        # URG is the only bit set, without the expected accompanying ACK
        $IPTABLES -A INPUT -p tcp --tcp-flags ACK,URG URG -j ScanD

        # SYN-Flood
        # (Request for new connection; large number indicate possible DDoS-type attack;
        #  same as --syn)
        $IPTABLES -A INPUT -p tcp --tcp-flags SYN,RST,ACK SYN -j DDoS

	  IPSYSCTL
	  TOS

        # Disallow fragmented packets.  This may not be as necessary as it once was.
        # Comment it out with # if desired.
#       $IPTABLES -A INPUT -f -i $EXTERNAL_INTERFACE -j LnD
#       $IPTABLES -A INPUT -f -i $INTERNAL_INTERFACE -j LnD


        #
        # Loopback
        #

        # Unlimited traffic on the loopback interface (lo)

        $IPTABLES -A INPUT -i $LOOPBACK_INTERFACE -j ACCEPT
        $IPTABLES -A OUTPUT -o $LOOPBACK_INTERFACE -j ACCEPT

        #
        # Masquerade internal traffic
        #

        if [ $MASQUERADING -gt 0 ]; then


            # All internal traffic is masqueraded externally

            $IPTABLES -t nat -A POSTROUTING -s $INTERNAL_NETWORK -o $EXTERNAL_INTERFACE -j SNAT \
            --to $EXTERNAL_IP

            # Enable IP Forwarding

            echo 1 >/proc/sys/net/ipv4/ip_forward

            #
            # Unlimited traffic within the local network
            #

            # All internal machines have access to the firewall machine

            $IPTABLES -A INPUT -i $INTERNAL_INTERFACE -s $INTERNAL_NETWORK -j ACCEPT

            $IPTABLES -A OUTPUT -o $INTERNAL_INTERFACE -d $INTERNAL_NETWORK -j ACCEPT

            if [ $VERBOSE -gt 0 ]; then
                echo "firewall: Masquerading internal network"
            fi
        fi

#
# SSH (see config) - Allowing remote client access to your local SSH server
#

        SSH_HI_PORTS="513:1023"                 # SSH Simultaneous Connections

        if [ $SSH_SERVER -gt 0 ]; then
        $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -p TCP  -m state --state NEW \
                --sport $UNPRIVPORTS --dport $SSH_PORT -s $MY_SSH_CLIENTS -d $EXTERNAL_IP -j SSHnD

        $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -p TCP  \
                --sport $UNPRIVPORTS --dport $SSH_PORT -s $MY_SSH_CLIENTS -d $EXTERNAL_IP -j ACCEPT

        $IPTABLES -A OUTPUT -o $EXTERNAL_INTERFACE -p TCP -m state --state ESTABLISHED,RELATED \
                --sport $SSH_PORT --dport $UNPRIVPORTS -s $EXTERNAL_IP -d $MY_SSH_CLIENTS -j ACCEPT

        $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -p TCP  \
                --sport $SSH_HI_PORTS --dport $SSH_PORT -s $MY_SSH_CLIENTS -d $EXTERNAL_IP -j ACCEPT

        $IPTABLES -A OUTPUT -o $EXTERNAL_INTERFACE -p TCP -m state --state ESTABLISHED,RELATED \
                --sport $SSH_PORT --dport $SSH_HI_PORTS -s $EXTERNAL_IP -d $MY_SSH_CLIENTS -j ACCEPT

        if [ $VERBOSE -gt 0 ]; then
                echo "firewall: Remote sites may access local SSH server"
        fi

        fi

        #
        # Refuse any connections to/from problem sites.
        #
        # /etc/firewall/firewall.banned contains a list of IPs
        # to block all access, both inbound and outbound.
        # The file should contain IP addresses with CIDR
        # netmask, one per line:
        #
        # NOTE: No comments are allowed in the file.
        #
        # 111.222.333.444/32            - To block a single IP address
        # 111.222.333.444/8             - To block a Class-A network
        # 111.222.333.444/16            - To block a Class-B network
        # 111.222.333.444/24            - To block a Class-C network
        #
        # The CIDR netmask number describes the number of bits
        # in the network portion of the address, and may be on
        # any boundary.
        #

        if [ -f /etc/firewall/firewall.banned ]; then
            while read BANNED; do
                $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -s $BANNED -j Banned
                $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -d $BANNED -j Banned
                $IPTABLES -A OUTPUT -o $EXTERNAL_INTERFACE -s $BANNED -j Banned
                $IPTABLES -A OUTPUT -o $EXTERNAL_INTERFACE -d $BANNED -j Banned
            done < /etc/firewall/firewall.banned
            echo "firewall: Banned addresses added to rule set"
        fi


        #
        # Refuse connections from IANA-reserved blocks
        #

        if [ -f /etc/firewall/firewall.iana-reserved ]; then
            while read RESERVED; do
                $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -s $RESERVED -j IANA
                $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -d $RESERVED -j IANA
                $IPTABLES -A OUTPUT -o $EXTERNAL_INTERFACE -s $RESERVED -j IANA
                $IPTABLES -A OUTPUT -o $EXTERNAL_INTERFACE -d $RESERVED -j IANA
            done < /etc/firewall/firewall.iana-reserved
            echo "firewall: Connections from IANA-reserved addresses blocked"
        fi


        #
        # Localizations
        #
        # The /etc/firewall/firewall.local file should contain rules in
        # standard '$IPTABLES' format.
        #

        if [ -f /etc/firewall/firewall.local.$IPTABLES ]; then
            . /etc/firewall/firewall.local.$IPTABLES
            echo "firewall: Local rules added"
        fi



        #
        # ICMP
        #

        # (4) Source Quench.
        # Incoming & outgoing requests to slow down (flow control)

        $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -p ICMP --icmp-type 4 \
            -s $ANYWHERE  -d $EXTERNAL_IP -j ACCEPT

        $IPTABLES -A OUTPUT -o $EXTERNAL_INTERFACE -p ICMP --icmp-type 4 \
            -s $EXTERNAL_IP  -d $ANYWHERE -j ACCEPT

        if [ $MASQUERADING -gt 0 ]; then
            $IPTABLES -A FORWARD -p ICMP --icmp-type 4 -j ACCEPT
        fi

        # (12) Parameter Problem.
        # Incoming & outgoing error messages

        $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -p ICMP --icmp-type 12 \
            -s $ANYWHERE  -d $EXTERNAL_IP -j ACCEPT

        $IPTABLES -A OUTPUT -o $EXTERNAL_INTERFACE -p ICMP --icmp-type 12 \
            -s $EXTERNAL_IP  -d $ANYWHERE -j ACCEPT

        if [ $MASQUERADING -gt 0 ]; then
            $IPTABLES -A FORWARD -p ICMP --icmp-type 12 -j ACCEPT
        fi

        # (3) Destination Unreachable, Service Unavailable.
        # Incoming & outgoing size negotiation, service or
        # destination unavailability, final traceroute response

        $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -p ICMP --icmp-type 3 \
            -s $ANYWHERE  -d $EXTERNAL_IP -j ACCEPT

        $IPTABLES -A OUTPUT -o $EXTERNAL_INTERFACE -p ICMP --icmp-type 3 \
            -s $EXTERNAL_IP  -d $ANYWHERE -j ACCEPT

        $IPTABLES -A OUTPUT -o $EXTERNAL_INTERFACE -p ICMP --icmp-type \
            fragmentation-needed -s $EXTERNAL_IP  -d $ANYWHERE -j ACCEPT

        if [ $MASQUERADING -gt 0 ]; then
            $IPTABLES -A FORWARD -p ICMP --icmp-type 3 -j ACCEPT
            $IPTABLES -A FORWARD -p ICMP --icmp-type fragmentation-needed -j ACCEPT
        fi

        # (11) Time Exceeded.
        # Incoming & outgoing timeout conditions,
        # also intermediate TTL response to traceroutes

        $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -p ICMP --icmp-type 11 \
            -s $ANYWHERE  -d $EXTERNAL_IP -j ACCEPT

        $IPTABLES -A OUTPUT -o $EXTERNAL_INTERFACE -p ICMP --icmp-type 11 \
            -s $EXTERNAL_IP  -d $ANYWHERE -j ACCEPT

        if [ $MASQUERADING -gt 0 ]; then
            $IPTABLES -A FORWARD -p ICMP --icmp-type 11 -j ACCEPT
        fi

        # (0 | 8) Allow OUTPUT pings to anywhere.

        if [ $OUTBOUND_PING -gt 0 ]; then

            $IPTABLES -A OUTPUT -o $EXTERNAL_INTERFACE -p ICMP --icmp-type 8 \
                -s $EXTERNAL_IP  -d $ANYWHERE -j ACCEPT

            $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -p ICMP --icmp-type 0 \
                -s $ANYWHERE  -d $EXTERNAL_IP -j ACCEPT

            if [ $MASQUERADING -gt 0 ]; then
               $IPTABLES -A FORWARD -p ICMP --icmp-type 8 -s $INTERNAL_NETWORK -j ACCEPT
               $IPTABLES -A FORWARD -p ICMP --icmp-type 0 -d $INTERNAL_NETWORK -j ACCEPT
            fi

            if [ $VERBOSE -gt 0 ]; then
                echo "firewall: Outbound ping enabled"
            fi

        fi

        # (0 | 8) Allow incoming pings from anywhere
        #       (stops at firewall).

        if [ $INBOUND_PING -gt 0 ]; then

            $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -p ICMP --icmp-type 8 \
                -s $ANYWHERE  -d $EXTERNAL_IP -j ACCEPT

            $IPTABLES -A OUTPUT -o $EXTERNAL_INTERFACE -p ICMP --icmp-type 0 \
                -s $EXTERNAL_IP  -d $ANYWHERE -j ACCEPT

            if [ $VERBOSE -gt 0 ]; then
                echo "firewall: Inbound ping enabled"
            fi

        fi

        #
        # Unprivileged Ports
        # Avoid ports subject to protocol and system administration problems.
        #

        NFS_PORT="2049"                         # (TCP/UDP) NFS
        OPENWINDOWS_PORT="2000"                 # (TCP) Openwindows
        SOCKS_PORT="1080"                       # (TCP) Socks


        # Openwindows: establishing a connection

        $IPTABLES -A OUTPUT -o $EXTERNAL_INTERFACE -p TCP -m state --state NEW \
        --dport $OPENWINDOWS_PORT -s $EXTERNAL_IP -d $ANYWHERE -j LnR

        # Openwindows: incoming connection

        $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -p TCP -m state --state NEW \
        --dport $OPENWINDOWS_PORT -d $EXTERNAL_IP -j LnD

        # X Window: establishing a remote connection

        $IPTABLES -A OUTPUT -o $EXTERNAL_INTERFACE -p TCP -m state --state NEW \
        --dport $XWINDOW_PORTS -s $EXTERNAL_IP -d $ANYWHERE -j LnR

        # X Window: incoming connection attempt

        $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -p TCP -m state --state NEW \
        --dport $XWINDOW_PORTS -d $EXTERNAL_IP -j LnD

        # SOCKS: establishing a connection

        $IPTABLES -A OUTPUT -o $EXTERNAL_INTERFACE -p TCP -m state --state NEW \
        --dport $SOCKS_PORT -s $EXTERNAL_IP -d $ANYWHERE  -j LnR

        # SOCKS: incoming connection

        $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -p TCP -m state --state NEW \
        --dport $SOCKS_PORT -d $EXTERNAL_IP -j LnD

        # NFS: TCP connections

        $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -p TCP -m state --state NEW \
        --dport $NFS_PORT -d $EXTERNAL_IP -j LnD

        $IPTABLES -A OUTPUT -o $EXTERNAL_INTERFACE -p TCP -m state --state NEW \
        --dport $NFS_PORT -d $ANYWHERE -j LnR

        # NFS: UDP connections

        $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -p UDP \
        --dport $NFS_PORT -d $EXTERNAL_IP -j LnD

        # NFS: incoming request (normal UDP mode)

        $IPTABLES -A OUTPUT -o $EXTERNAL_INTERFACE -p UDP \
        --dport $NFS_PORT -d $ANYWHERE -j LnR


        #
        # DNAT/SNAT Port Forwarding
        #

        if [ $PORT_FORWARD -gt 0 ]; then
           if [ -f /etc/firewall/firewall.nat ]; then
             while read IP_PORT; do
               # extract the protocols, IPs and ports
               NAT_TYPE=$(echo "$IP_PORT" | awk '{print $1}')
               NAT_EXT_PORT=$(echo "$IP_PORT" | awk '{print $2}')
               NAT_INT_IP=$(echo "$IP_PORT" | awk '{print $3}')
               NAT_INT_PORT=$(echo "$IP_PORT" | awk '{print $4}')

               # write the rules!

               # this is the prerouting dnat
               $IPTABLES -A PREROUTING -t nat -p $NAT_TYPE -d $EXTERNAL_IP --dport $NAT_EXT_PORT -j DNAT \
                --to-destination $NAT_INT_IP:$NAT_INT_PORT

               # This allows packets from external->internal
               $IPTABLES -A FORWARD -i $EXTERNAL_INTERFACE -o $INTERNAL_INTERFACE -p $NAT_TYPE  \
                -d $NAT_INT_IP --dport $NAT_INT_PORT -m state \
                --state NEW,ESTABLISHED,RELATED -j ACCEPT

               # This allows packets from internal->external
               $IPTABLES -A FORWARD -i $INTERNAL_INTERFACE -o $EXTERNAL_INTERFACE -p $NAT_TYPE  \
                -s $NAT_INT_IP --sport $NAT_INT_PORT -m state \
                --state NEW,ESTABLISHED,RELATED -j ACCEPT

               # This enables access to the 'public' server from the internal network
               $IPTABLES -t nat -A POSTROUTING -d $NAT_INT_IP -s $INTERNAL_NETWORK \
                -p $NAT_TYPE --dport $NAT_INT_PORT -j SNAT --to $INTERNAL_IP

               echo firewall: dnat: $NAT_TYPE:$EXTERNAL_IP:$NAT_EXT_PORT - $NAT_INT_IP:$NAT_INT_PORT

             done < /etc/firewall/firewall.nat

            # unset some variables
              unset IP_PORT
              unset NAT_TYPE
              unset NAT_EXT_PORT
              unset NAT_INT_IP
              unset NAT_INT_PORT

           else
              echo "firewall.nat (port-forwarding table) not found!  Port-forwarding not enabled."
           fi
        fi


        #
        # NOTE:
        #     The symbolic names used in /etc/services for the port numbers
        #     vary by supplier.
        #

        # Required Services

        #
        # DNS client modes (53)
        #

        if [ $DNS_CLIENT -gt 0 ]; then

            $IPTABLES -A OUTPUT -o $EXTERNAL_INTERFACE -p UDP \
                --sport $UNPRIVPORTS --dport 53 -s $EXTERNAL_IP \
                -d $ANYWHERE -j ACCEPT

            $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -m state --state ESTABLISHED,RELATED -p UDP --sport 53 \
                --dport $UNPRIVPORTS -s $ANYWHERE -d $EXTERNAL_IP -j ACCEPT

            if [ $MASQUERADING -gt 0 ]; then
               $IPTABLES -A FORWARD -p UDP -s $INTERNAL_NETWORK --sport $UNPRIVPORTS --dport 53 -j ACCEPT
               $IPTABLES -A FORWARD -p UDP -d $INTERNAL_NETWORK --sport 53 --dport $UNPRIVPORTS -j ACCEPT
            fi

            # TCP client-to-server requests are allowed by the protocol
            # if UDP requests fail. This is rarely seen. Usually, clients
            # use TCP as a secondary name server for zone transfers from
            # their primary name servers, and as hackers.

            $IPTABLES -A OUTPUT -o $EXTERNAL_INTERFACE -p TCP --sport \
                $UNPRIVPORTS --dport 53 -s $EXTERNAL_IP -d $ANYWHERE -j ACCEPT

            $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -p TCP -m state --state ESTABLISHED,RELATED \
                --sport 53 --dport $UNPRIVPORTS -s $ANYWHERE -d $EXTERNAL_IP -j ACCEPT

            if [ $MASQUERADING -gt 0 ]; then
               $IPTABLES -A FORWARD -p TCP -s $INTERNAL_NETWORK --sport $UNPRIVPORTS --dport 53 -j ACCEPT
               $IPTABLES -A FORWARD -m state --state ESTABLISHED,RELATED -p TCP -d $INTERNAL_NETWORK \
                --sport 53 --dport $UNPRIVPORTS -j ACCEPT
            fi

            if [ $VERBOSE -gt 0 ]; then
                echo "firewall: DNS client enabled"
            fi
        fi

        #
        # DNS server modes (53)
        #

        #
        # DNS caching & forwarding name server
        #

        if [ $DNS_CACHING_SERVER -gt 0 ]; then

            # Server-to-server query or response
            # Caching only name server uses UDP, not TCP

            $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -p UDP \
        --sport 53 --dport 53 -s $ANYWHERE -d $EXTERNAL_IP -j ACCEPT

            $IPTABLES -A OUTPUT -o $EXTERNAL_INTERFACE -p UDP \
        --sport 53 --dport 53 -s $EXTERNAL_IP -d $ANYWHERE -j ACCEPT

            if [ $VERBOSE -gt 0 ]; then
                echo "firewall: DNS Caching server enabled"
            fi

        fi

        #
        # DNS full name server
        #

        if [ $DNS_FULL_SERVER -gt 0 ]; then

            # Client-to-server DNS transaction.

            $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -p UDP \
        --sport $UNPRIVPORTS --dport 53 -s $ANYWHERE -d $EXTERNAL_IP -j ACCEPT

            $IPTABLES -A OUTPUT -o $EXTERNAL_INTERFACE -p UDP \
        --sport 53 --dport $UNPRIVPORTS -s $EXTERNAL_IP -d $ANYWHERE -j ACCEPT

            # Peer-to-peer server DNS transaction.

            $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -p UDP \
        --sport 53 --dport 53 -s $ANYWHERE -d $EXTERNAL_IP -j ACCEPT

            $IPTABLES -A OUTPUT -o $EXTERNAL_INTERFACE -p UDP \
        --sport 53 --dport 53 -s $EXTERNAL_IP -d $ANYWHERE -j ACCEPT

            # Zone Transfers.
            # Due to the potential danger of zone transfers,
            # allow TCP traffic to only specific secondaries.

            # /etc/firewall/firewall.dns contains a list of
            # secondary, tertiary, etc. domain name servers with which
            # zone transfers are allowed.  The file should contain IP
            # addresses with CIDR netmask, one per line:


        if [ -f /etc/firewall/firewall.dns ]; then
                while read DNS_SECONDARY; do

            $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -p TCP \
                --sport $UNPRIVPORTS --dport 53 -s $DNS_SECONDARY -d $EXTERNAL_IP -j ACCEPT

            $IPTABLES -A OUTPUT -o $EXTERNAL_INTERFACE -p TCP -m state --state ESTABLISHED,RELATED \
                --sport 53 --dport $UNPRIVPORTS -s $EXTERNAL_IP -d $DNS_SECONDARY -j ACCEPT

            done < /etc/firewall/firewall.dns

        else
            echo "firewall: ** No secondary DNS configured **"

        fi

            if [ $VERBOSE -gt 0 ]; then
                echo "firewall: DNS Full server enabled"
            fi

        fi

        #
        # AUTH (113) - Allowing your outgoing AUTH requests as a client
        #

        if [ $AUTH_CLIENT -gt 0 ]; then

            $IPTABLES -A OUTPUT -o $EXTERNAL_INTERFACE -p TCP \
        --sport $UNPRIVPORTS --dport 113 -s $EXTERNAL_IP -d $ANYWHERE -j ACCEPT

            $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -p TCP -m state --state ESTABLISHED,RELATED \
        --sport 113 --dport $UNPRIVPORTS -s $ANYWHERE -d $EXTERNAL_IP -j ACCEPT

            if [ $MASQUERADING -gt 0 ]; then
               $IPTABLES -A FORWARD -p TCP -s $INTERNAL_NETWORK --sport $UNPRIVPORTS --dport 113 -j ACCEPT
               $IPTABLES -A FORWARD -m state --state ESTABLISHED,RELATED -p TCP -d $INTERNAL_NETWORK \
                --sport 113 --dport $UNPRIVPORTS -j ACCEPT
            fi

            if [ $VERBOSE -gt 0 ]; then
                echo "firewall: Auth client enabled"
            fi

        fi


        # AUTH server (113)

        if [ $AUTH_SERVER -gt 0 ]; then

            # Accepting incoming AUTH requests

            $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -p TCP \
        --sport $UNPRIVPORTS --dport 113 -s $ANYWHERE -d $EXTERNAL_IP -j ACCEPT

            $IPTABLES -A OUTPUT -o $EXTERNAL_INTERFACE -p TCP -m state --state ESTABLISHED,RELATED \
        --sport 113 --dport $UNPRIVPORTS -s $EXTERNAL_IP -d $ANYWHERE -j ACCEPT

            if [ $VERBOSE -gt 0 ]; then
                echo "firewall: Auth server enabled"
            fi

        else

            # Rejecting incoming AUTH requests

            $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -p TCP \
                --dport 113 -d $EXTERNAL_IP -j LnR

            if [ $VERBOSE -gt 0 ]; then
                echo "firewall: Auth server requests will be rejected"
            fi

        fi


        #
        # TCP Services on selected ports.
        #

        #
        # Sending Mail through a remote SMTP server (25)
        #

        if [ $SMTP_REMOTE_SERVER -gt 0 ]; then

            # SMTP client to an ISP account without a local server

            $IPTABLES -A OUTPUT -o $EXTERNAL_INTERFACE -p TCP \
        --sport $UNPRIVPORTS --dport 25 -s $EXTERNAL_IP -d $SMTP_SERVER -j ACCEPT

            $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -p TCP -m state --state ESTABLISHED,RELATED \
        --sport 25 --dport $UNPRIVPORTS -s $SMTP_SERVER -d $EXTERNAL_IP -j ACCEPT

            if [ $MASQUERADING -gt 0 ]; then
               $IPTABLES -A FORWARD -p TCP -s $INTERNAL_NETWORK --sport $UNPRIVPORTS --dport 25 -j ACCEPT
               $IPTABLES -A FORWARD -m state --state ESTABLISHED,RELATED -p TCP -d $INTERNAL_NETWORK \
                --sport 25 --dport $UNPRIVPORTS -j ACCEPT
            fi

            if [ $VERBOSE -gt 0 ]; then
                echo "firewall: SMTP Remote server enabled"
            fi
        fi

        #
        # Sending Mail through a local SMTP server (25)
        #

        if [ $SMTP_LOCAL_SERVER -gt 0 ]; then
	   if [ $SMTP_LOCAL_SEND -gt 0 ]; then
            $IPTABLES -A OUTPUT -o $EXTERNAL_INTERFACE -p TCP \
	        --sport $UNPRIVPORTS --dport 25 -s $EXTERNAL_IP -d $ANYWHERE -j ACCEPT
            $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -p TCP -m state --state ESTABLISHED,RELATED \
	        --sport 25 --dport $UNPRIVPORTS -s $ANYWHERE -d $EXTERNAL_IP -j ACCEPT

              if [ $VERBOSE -gt 0 ]; then
                   echo "firewall: SMTP Remote server sending enabled"
               fi
							       
	   fi
            # Receiving Mail as a Local SMTP server (25)

            $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -p TCP \
        --sport $UNPRIVPORTS --dport 25 -s $ANYWHERE -d $EXTERNAL_IP -j ACCEPT

            $IPTABLES -A OUTPUT -o $EXTERNAL_INTERFACE -p TCP -m state --state ESTABLISHED,RELATED \
        --sport 25 --dport $UNPRIVPORTS -s $EXTERNAL_IP -d $ANYWHERE -j ACCEPT

            if [ $VERBOSE -gt 0 ]; then
                echo "firewall: SMTP Local server enabled"
            fi

        fi


        #
        # SKYPE (33033)
        #

        if [ $SKYPE_CLIENT -gt 0 ]; then

            $IPTABLES -A OUTPUT -o $EXTERNAL_INTERFACE -p TCP \
        --sport $UNPRIVPORTS --dport 1863 -s $EXTERNAL_IP -d $ANYWHERE -j ACCEPT

            $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -p TCP -m state --state ESTABLISHED,RELATED \
        --sport 1863 --dport $UNPRIVPORTS -s $ANYWHERE -d $EXTERNAL_IP -j ACCEPT

            if [ $MASQUERADING -gt 0 ]; then
               $IPTABLES -A FORWARD -p TCP -s $INTERNAL_NETWORK --sport $UNPRIVPORTS --dport 1863 -j ACCEPT
               $IPTABLES -A FORWARD -m state --state ESTABLISHED,RELATED -p TCP -d $INTERNAL_NETWORK \
                --sport 1863 --dport $UNPRIVPORTS -j ACCEPT
            fi

            $IPTABLES -A OUTPUT -o $EXTERNAL_INTERFACE -p UDP \
        --sport $UNPRIVPORTS --dport 46704 -s $EXTERNAL_IP -d $ANYWHERE -j ACCEPT

            $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -p UDP -m state --state ESTABLISHED,RELATED \
        --sport 46704 --dport $UNPRIVPORTS -s $ANYWHERE -d $EXTERNAL_IP -j ACCEPT

            if [ $MASQUERADING -gt 0 ]; then
               $IPTABLES -A FORWARD -p UDP -s $INTERNAL_NETWORK --sport $UNPRIVPORTS --dport 46704 -j ACCEPT
               $IPTABLES -A FORWARD -m state --state ESTABLISHED,RELATED -p UDP -d $INTERNAL_NETWORK \
                --sport 46704 --dport $UNPRIVPORTS -j ACCEPT
            fi

            if [ $VERBOSE -gt 0 ]; then
                echo "firewall: Clients may access remote SKYPE servers"
            fi

        fi



        #
        # POP3 (110) - Retrieving Mail as a POP3 client
        #

        if [ $POP3_CLIENT -gt 0 ]; then

            $IPTABLES -A OUTPUT -o $EXTERNAL_INTERFACE -p TCP \
        --sport $UNPRIVPORTS --dport 110 -s $EXTERNAL_IP -d $POP_SERVER -j ACCEPT

            $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -p TCP -m state --state ESTABLISHED,RELATED \
        --sport 110 --dport $UNPRIVPORTS -s $POP_SERVER -d $EXTERNAL_IP -j ACCEPT

            if [ $MASQUERADING -gt 0 ]; then
               $IPTABLES -A FORWARD -p TCP -s $INTERNAL_NETWORK --sport $UNPRIVPORTS --dport 110 -j ACCEPT
               $IPTABLES -A FORWARD -m state --state ESTABLISHED,RELATED -p TCP -d $INTERNAL_NETWORK \
                --sport 110 --dport $UNPRIVPORTS -j ACCEPT
            fi

            if [ $VERBOSE -gt 0 ]; then
                echo "firewall: Clients may access remote POP-3 servers"
            fi

        fi



        #
        # POP3 (110) - Hosting a POP3 server for remote clients
        #

        if [ $POP3_SERVER -gt 0 ]; then

            $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -p TCP \
        --sport $UNPRIVPORTS --dport 110 -s $MY_POP3_CLIENTS -d $EXTERNAL_IP -j ACCEPT

            $IPTABLES -A OUTPUT -o $EXTERNAL_INTERFACE -p TCP -m state --state ESTABLISHED,RELATED \
        --sport 110 --dport $UNPRIVPORTS -s $EXTERNAL_IP -d $MY_POP3_CLIENTS -j ACCEPT

            if [ $VERBOSE -gt 0 ]; then
                echo "firewall: Remote clients may access local POP-3 server"
            fi

        fi

        #
        # IMAP (143) - Retrieving Mail as an IMAP client
        #

        if [ $IMAP_CLIENT -gt 0 ]; then

            $IPTABLES -A OUTPUT -o $EXTERNAL_INTERFACE -p TCP \
        --sport $UNPRIVPORTS --dport 143 -s $EXTERNAL_IP -d $MY_IMAP_SERVER -j ACCEPT

            $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -p TCP -m state --state ESTABLISHED,RELATED \
        --sport 143 --dport $UNPRIVPORTS -s $MY_IMAP_SERVER -d $EXTERNAL_IP -j ACCEPT

            if [ $MASQUERADING -gt 0 ]; then
               $IPTABLES -A FORWARD -p TCP -s $INTERNAL_NETWORK --sport $UNPRIVPORTS --dport 143 -j ACCEPT
               $IPTABLES -A FORWARD -m state --state ESTABLISHED,RELATED -p TCP -d $INTERNAL_NETWORK \
                --sport 143 --dport $UNPRIVPORTS -j ACCEPT
            fi

            if [ $VERBOSE -gt 0 ]; then
                echo "firewall: Clients may access remote IMAP servers"
            fi

        fi

        #
        # IMAP (143) - Hosting an IMAP server for remote clients
        #

        if [ $IMAP_SERVER -gt 0 ]; then

            $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -p TCP \
        --sport $UNPRIVPORTS --dport 143 -s $MY_IMAP_CLIENTS -d $EXTERNAL_IP -j ACCEPT

            $IPTABLES -A OUTPUT -o $EXTERNAL_INTERFACE -p TCP -m state --state ESTABLISHED,RELATED \
        --sport 143 --dport $UNPRIVPORTS -s $EXTERNAL_IP -d $MY_IMAP_CLIENTS -j ACCEPT

            if [ $VERBOSE -gt 0 ]; then
                echo "firewall: Remote clients may access local IMAP server"
            fi

        fi


        #
        # NNTP (119) - Reading and posting news as a Usenet client
        #

        if [ $NNTP_CLIENT -gt 0 ]; then

            $IPTABLES -A OUTPUT -o $EXTERNAL_INTERFACE -p TCP \
        --sport $UNPRIVPORTS --dport 119 -s $EXTERNAL_IP -d $NEWS_SERVER -j ACCEPT

            $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -p TCP -m state --state ESTABLISHED,RELATED \
        --sport 119 --dport $UNPRIVPORTS -s $NEWS_SERVER -d $EXTERNAL_IP -j ACCEPT

            if [ $MASQUERADING -gt 0 ]; then
               $IPTABLES -A FORWARD -p TCP -s $INTERNAL_NETWORK --sport $UNPRIVPORTS --dport 119 -j ACCEPT
               $IPTABLES -A FORWARD -m state --state ESTABLISHED,RELATED -p TCP -d $INTERNAL_NETWORK \
                --sport 119 --dport $UNPRIVPORTS -j ACCEPT
            fi

            if [ $VERBOSE -gt 0 ]; then
                echo "firewall: Clients may access remote NNTP servers"
            fi

        fi

        #
        # NNTP (119) - Hosting a Usenet news server for remote clients
        #

        if [ $NNTP_SERVER -gt 0 ]; then

            $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -p TCP \
        --sport $UNPRIVPORTS --dport 119 -s $MY_NNTP_CLIENTS -d $EXTERNAL_IP -j ACCEPT

            $IPTABLES -A OUTPUT -o $EXTERNAL_INTERFACE -p TCP -m state --state ESTABLISHED,RELATED \
        --sport 119 --dport $UNPRIVPORTS -s $EXTERNAL_IP -d $MY_NNTP_CLIENTS -j ACCEPT

            if [ $VERBOSE -gt 0 ]; then
                echo "firewall: Remote clients may access local NNTP server"
            fi

        fi

        #
        # NNTP (119) - Allowing peer news feeds for a local Usenet server
        #

        if [ $NNTP_NEWS_FEED -gt 0 ]; then

            $IPTABLES -A OUTPUT -o $EXTERNAL_INTERFACE -p TCP \
        --sport $UNPRIVPORTS --dport 119 -s $EXTERNAL_IP -d $MY_NEWS_FEED -j ACCEPT

            $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -p TCP -m state --state ESTABLISHED,RELATED \
        --sport 119 --dport $UNPRIVPORTS -s $MY_NEWS_FEED -d $EXTERNAL_IP -j ACCEPT

            if [ $VERBOSE -gt 0 ]; then
                echo "firewall: External NNTP News feed access enabled"
            fi

        fi

        #
        # Secure NNTP (563) - Reading and posting news as a Usenet client over SSL
        # Submitted by Renaud Colinet
        #

        if [ $NNTPS_CLIENT -gt 0 ]; then

            $IPTABLES -A OUTPUT -o $EXTERNAL_INTERFACE -p TCP \
        --sport $UNPRIVPORTS --dport 563 -s $EXTERNAL_IP -d $SNEWS_SERVER -j ACCEPT

            $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -p TCP -m state --state ESTABLISHED,RELATED \
        --sport 563 --dport $UNPRIVPORTS -s $SNEWS_SERVER -d $EXTERNAL_IP -j ACCEPT

            if [ $MASQUERADING -gt 0 ]; then
               $IPTABLES -A FORWARD -p TCP -s $INTERNAL_NETWORK --sport $UNPRIVPORTS --dport 563 -j ACCEPT
               $IPTABLES -A FORWARD -m state --state ESTABLISHED,RELATED -p TCP -d $INTERNAL_NETWORK \
                --sport 563 --dport $UNPRIVPORTS -j ACCEPT
            fi

            if [ $VERBOSE -gt 0 ]; then
                echo "firewall: Clients may access remote secure NNTP servers"
            fi

        fi

        #
        # TELNET (23) - Allowing outgoing client access to remote sites
        #

        if [ $TELNET_CLIENT -gt 0 ]; then

            $IPTABLES -A OUTPUT -o $EXTERNAL_INTERFACE -p TCP \
        --sport $UNPRIVPORTS --dport 23 -s $EXTERNAL_IP -d $ANYWHERE -j ACCEPT

            $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -p TCP -m state --state ESTABLISHED,RELATED \
        --sport 23 --dport $UNPRIVPORTS -s $ANYWHERE -d $EXTERNAL_IP -j ACCEPT

            if [ $MASQUERADING -gt 0 ]; then
               $IPTABLES -A FORWARD -p TCP -s $INTERNAL_NETWORK --sport $UNPRIVPORTS --dport 23 -j ACCEPT
               $IPTABLES -A FORWARD -m state --state ESTABLISHED,RELATED -p TCP -d $INTERNAL_NETWORK \
                --sport 23 --dport $UNPRIVPORTS -j ACCEPT
            fi

            if [ $VERBOSE -gt 0 ]; then
                echo "firewall: Clients may access remote TELNET servers"
            fi

        fi

        #
        # TELNET (23) - Allowing incoming access to your local server
        #

        if [ $TELNET_SERVER -gt 0 ]; then

            $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -p TCP \
        --sport $UNPRIVPORTS --dport 23 -s $MY_TELNET_CLIENTS -d $EXTERNAL_IP -j ACCEPT

            $IPTABLES -A OUTPUT -o $EXTERNAL_INTERFACE -p TCP -m state --state ESTABLISHED,RELATED \
        --sport 23 --dport $UNPRIVPORTS -s $EXTERNAL_IP -d $MY_TELNET_CLIENTS -j ACCEPT

            if [ $VERBOSE -gt 0 ]; then
                echo "firewall: Remote clients may access local TELNET server"
            fi

        fi

        #
        # SSH Client (22) - Allowing client access to remote SSH servers
        #

        if [ $SSH_CLIENT -gt 0 ]; then

            $IPTABLES -A OUTPUT -o $EXTERNAL_INTERFACE -p TCP \
        --sport $UNPRIVPORTS --dport 22 -s $EXTERNAL_IP -d $ANYWHERE -j ACCEPT

            $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -p TCP -m state --state ESTABLISHED,RELATED \
        --sport 22 --dport $UNPRIVPORTS -s $ANYWHERE -d $EXTERNAL_IP -j ACCEPT

            $IPTABLES -A OUTPUT -o $EXTERNAL_INTERFACE -p TCP \
        --sport $SSH_HI_PORTS --dport 22 -s $EXTERNAL_IP -d $ANYWHERE -j ACCEPT

            $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -p TCP -m state --state ESTABLISHED,RELATED \
        --sport 22 --dport $SSH_HI_PORTS -s $ANYWHERE -d $EXTERNAL_IP -j ACCEPT

            if [ $MASQUERADING -gt 0 ]; then
               $IPTABLES -A FORWARD -p TCP -s $INTERNAL_NETWORK --sport $UNPRIVPORTS --dport 22 -j ACCEPT
               $IPTABLES -A FORWARD -m state --state ESTABLISHED,RELATED -p TCP -d $INTERNAL_NETWORK \
                --sport 22 --dport $UNPRIVPORTS -j ACCEPT
               $IPTABLES -A FORWARD -p TCP -s $INTERNAL_NETWORK --sport $SSH_HI_PORTS --dport 22 -j ACCEPT
               $IPTABLES -A FORWARD -m state --state ESTABLISHED,RELATED -p TCP -d $INTERNAL_NETWORK \
                --sport 22 --dport $SSH_HI_PORTS -j ACCEPT
            fi

            if [ $VERBOSE -gt 0 ]; then
                echo "firewall: Clients may access remote SSH servers"
            fi

        fi


        #
        # FTP (20, 21) - Allowing outgoing client access to remote FTP servers
        #

        if [ $FTP_CLIENT -gt 0 ]; then

            # Outgoing request

            $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -p TCP -m state --state ESTABLISHED,RELATED \
        --sport 21 --dport $UNPRIVPORTS -s $ANYWHERE -d $EXTERNAL_IP -j ACCEPT

            $IPTABLES -A OUTPUT -o $EXTERNAL_INTERFACE -p TCP -m state --state NEW,ESTABLISHED \
        --sport $UNPRIVPORTS --dport 21 -s $EXTERNAL_IP -d $ANYWHERE -j ACCEPT

            # Normal Port mode FTP data channels

            $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -p TCP -m state --state NEW \
        --sport 20 --dport $UNPRIVPORTS -s $ANYWHERE -d $EXTERNAL_IP -j ACCEPT

            $IPTABLES -A OUTPUT -o $EXTERNAL_INTERFACE -p TCP -m state --state ESTABLISHED,RELATED \
        --sport $UNPRIVPORTS --dport 20 -s $EXTERNAL_IP -d $ANYWHERE -j ACCEPT

            # Passive mode FTP data channels

            $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -p TCP -m state --state ESTABLISHED,RELATED \
        --sport $UNPRIVPORTS --dport $UNPRIVPORTS -s $ANYWHERE -d $EXTERNAL_IP -j ACCEPT

            $IPTABLES -A OUTPUT -o $EXTERNAL_INTERFACE -p TCP -m state --state NEW,ESTABLISHED \
        --sport $UNPRIVPORTS --dport $UNPRIVPORTS -s $EXTERNAL_IP -d $ANYWHERE -j ACCEPT

            if [ $MASQUERADING -gt 0 ]; then
               $IPTABLES -A FORWARD -p TCP -s $INTERNAL_NETWORK --sport $UNPRIVPORTS --dport 20:21 -j ACCEPT
               $IPTABLES -A FORWARD -p TCP -d $INTERNAL_NETWORK --sport 20:21 --dport $UNPRIVPORTS -j ACCEPT
               $IPTABLES -A FORWARD -p TCP -s $INTERNAL_NETWORK --sport 10000:10010 --dport $UNPRIVPORTS -j ACCEPT
               $IPTABLES -A FORWARD -p TCP -d $INTERNAL_NETWORK --sport $UNPRIVPORTS --dport 10000:10010 -j ACCEPT
            fi

            if [ $VERBOSE -gt 0 ]; then
                echo "firewall: Clients may access remote FTP servers"
            fi

        fi

        #
        # FTP (20, 21) - Allowing incoming access to your local FTP server
        #

      if [ $FTP_SERVER -gt 0 ]; then

            # Incoming request

            $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -p TCP -m state --state NEW,ESTABLISHED \
        --sport $UNPRIVPORTS --dport 21 -s $MY_FTP_CLIENTS -d $EXTERNAL_IP -j ACCEPT

            $IPTABLES -A OUTPUT -o $EXTERNAL_INTERFACE -p TCP -m state --state ESTABLISHED,RELATED \
        --sport 21 --dport $UNPRIVPORTS -s $EXTERNAL_IP -d $MY_FTP_CLIENTS -j ACCEPT

            # Normal Port mode FTP data channel responses

            $IPTABLES -A OUTPUT -o $EXTERNAL_INTERFACE -p TCP \
        --sport 20 --dport $UNPRIVPORTS -s $EXTERNAL_IP -d $MY_FTP_CLIENTS -j ACCEPT

            $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -p TCP -m state --state ESTABLISHED,RELATED \
        --sport $UNPRIVPORTS --dport 20 -s $MY_FTP_CLIENTS -d $EXTERNAL_IP -j ACCEPT

            # Passive mode FTP data channel responses

            $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -p TCP -m state --state NEW,ESTABLISHED \
        --sport $UNPRIVPORTS --dport $UNPRIVPORTS -s $MY_FTP_CLIENTS -d $EXTERNAL_IP -j ACCEPT

            $IPTABLES -A OUTPUT -o $EXTERNAL_INTERFACE -p TCP -m state --state ESTABLISHED,RELATED \
        --sport $UNPRIVPORTS --dport $UNPRIVPORTS -s $EXTERNAL_IP -d $MY_FTP_CLIENTS -j ACCEPT

            if [ $VERBOSE -gt 0 ]; then
                echo "firewall: Remote clients may access local FTP server"
            fi
        fi


        #
        # HTTP (80) - Accessing remote web sites as a client
        #

        if [ $HTTP_CLIENT -gt 0 ]; then

            $IPTABLES -A OUTPUT -o $EXTERNAL_INTERFACE -p TCP \
        --sport $UNPRIVPORTS --dport 80 -s $EXTERNAL_IP -d $ANYWHERE -j ACCEPT

            $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -p TCP -m state --state ESTABLISHED,RELATED \
        --sport 80 --dport $UNPRIVPORTS -s $ANYWHERE -d $EXTERNAL_IP -j ACCEPT

	  # if you want a passthrough HTTP gateway, uncomment the four lines below. Personnaly, I
	  # use a proxy server and I dont want to let Windows softs trying to connect somewhere using
        # port 80
#            if [ $MASQUERADING -gt 0 ]; then
#               $IPTABLES -A FORWARD -p TCP -s $INTERNAL_NETWORK --sport $UNPRIVPORTS --dport 80 -j ACCEPT
#               $IPTABLES -A FORWARD -p TCP -d $INTERNAL_NETWORK --sport 80 --dport $UNPRIVPORTS -j ACCEPT
#            fi

            if [ $VERBOSE -gt 0 ]; then
                echo "firewall: Clients may access remote HTTP servers"
            fi

        fi

        #
        # HTTP (80) - Allowing remote access to a local web server
        #

        if [ $HTTP_SERVER -gt 0 ]; then
            $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -p TCP \
        --sport $UNPRIVPORTS --dport 80 -s $MY_HTTP_CLIENTS -d $EXTERNAL_IP -j ACCEPT

            $IPTABLES -A OUTPUT -o $EXTERNAL_INTERFACE -p TCP -m state --state ESTABLISHED,RELATED \
        --sport 80 --dport $UNPRIVPORTS -s $EXTERNAL_IP -d $MY_HTTP_CLIENTS -j ACCEPT

            $IPTABLES -A INPUT -i $INTERNAL_INTERFACE -p TCP \
        --sport $UNPRIVPORTS --dport 8080 -s $MY_HTTP_CLIENTS -d $INTERNAL_IP -j ACCEPT

            $IPTABLES -A OUTPUT -o $INTERNAL_INTERFACE -p TCP -m state --state ESTABLISHED,RELATED \
        --sport 8080 --dport $UNPRIVPORTS -s $INTERNAL_IP -d $MY_HTTP_CLIENTS -j ACCEPT

            if [ $VERBOSE -gt 0 ]; then
                echo "firewall: Remote clients may access local HTTP server"
            fi

        fi

        #
        # OPENVPN (5000:5050)
        #

        if [ $OPENVPN_CLIENT -gt 0 ]; then

            $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -p UDP \
        --sport $UNPRIVPORTS --dport 5000:5050 -s $ANYWHERE -d $EXTERNAL_IP -j ACCEPT

            $IPTABLES -A OUTPUT -o $EXTERNAL_INTERFACE -p UDP -m state --state ESTABLISHED,RELATED \
        --sport 5000:5050 --dport $UNPRIVPORTS -s $EXTERNAL_IP -d $ANYWHERE -j ACCEPT

            if [ $VERBOSE -gt 0 ]; then
                echo "openvpn: Remote clients may access local HTTP server"
            fi

        fi


         # Allow packets from TUN/TAP devices.
        $IPTABLES -A INPUT -i tun+ -j ACCEPT
        $IPTABLES -A FORWARD -i tun+ -j ACCEPT
        $IPTABLES -A INPUT -i tap+ -j ACCEPT
        $IPTABLES -A FORWARD -i tap+ -j ACCEPT
        $IPTABLES -A INPUT -i br0 -j ACCEPT
        $IPTABLES -A FORWARD -i br0 -j ACCEPT

        $IPTABLES -A INPUT -p tcp --sport 137:139 -i eth0 -j ACCEPT

        $IPTABLES -A OUTPUT -p tcp --sport 137:139 -o tap+ -j DROP
        $IPTABLES -A OUTPUT -p udp --sport 137:139 -o tap+ -j DROP


        $IPTABLES -A OUTPUT -p tcp --sport 445 -o tap+ -j DROP
        $IPTABLES -A INPUT -p tcp --dport 445 -i tap+ -j DROP


        #
        # HTTPS (443) - Accessing remote web sites over SSL as a client
        #

        if [ $HTTPS_CLIENT -gt 0 ]; then

            $IPTABLES -A OUTPUT -o $EXTERNAL_INTERFACE -p TCP \
        --sport $UNPRIVPORTS --dport 443 -s $EXTERNAL_IP -d $ANYWHERE -j ACCEPT

            $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -p TCP -m state --state ESTABLISHED,RELATED \
        --sport 443 --dport $UNPRIVPORTS -s $ANYWHERE -d $EXTERNAL_IP -j ACCEPT

            if [ $MASQUERADING -gt 0 ]; then
               $IPTABLES -A FORWARD -p TCP -s $INTERNAL_NETWORK --sport $UNPRIVPORTS --dport 443 -j ACCEPT
               $IPTABLES -A FORWARD -m state --state ESTABLISHED,RELATED -p TCP -d $INTERNAL_NETWORK \
                --sport 443 --dport $UNPRIVPORTS -j ACCEPT
            fi

            if [ $VERBOSE -gt 0 ]; then
                echo "firewall: Clients may access remote HTTPS servers"
            fi

        fi

        #
        # HTTPS (443) - Allowing remote access to a local SSL web server
        #

        if [ $HTTPS_SERVER -gt 0 ]; then

            $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -p TCP \
        --sport $UNPRIVPORTS --dport 443 -s $MY_HTTP_CLIENTS -d $EXTERNAL_IP -j ACCEPT

            $IPTABLES -A OUTPUT -o $EXTERNAL_INTERFACE -p TCP -m state --state ESTABLISHED,RELATED \
        --sport 443 --dport $UNPRIVPORTS -s $EXTERNAL_IP -d $MY_HTTP_CLIENTS -j ACCEPT

            if [ $VERBOSE -gt 0 ]; then
                echo "firewall: Remote clients may access local HTTPS server"
            fi

        fi

        #
        # HTTP Proxy Client (8008/8080)
        #

        if [ $HTTP_PROXY -gt 0 ]; then

            $IPTABLES -A OUTPUT -o $EXTERNAL_INTERFACE -p TCP \
        --sport $UNPRIVPORTS --dport $WEB_PROXY_PORT -s $EXTERNAL_IP -d $WEB_PROXY_SERVER -j ACCEPT

            $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -p TCP -m state --state ESTABLISHED,RELATED \
        --sport $WEB_PROXY_PORT --dport $UNPRIVPORTS -s $WEB_PROXY_SERVER -d $EXTERNAL_IP -j ACCEPT

            if [ $MASQUERADING -gt 0 ]; then
               $IPTABLES -A FORWARD -p TCP -s $INTERNAL_NETWORK --sport $UNPRIVPORTS --dport $WEB_PROXY_PORT -j ACCEPT
               $IPTABLES -A FORWARD -m state --state ESTABLISHED,RELATED -p TCP -d $INTERNAL_NETWORK \
                --sport $WEB_PROXY_PORT --dport $UNPRIVPORTS -j ACCEPT
            fi

            if [ $VERBOSE -gt 0 ]; then
                echo "firewall: Clients may access remote sites via HTTP Proxy Server"
            fi

        fi



        #
        # FINGER (79) - Accessing remote finger servers as a client
        #

        if [ $FINGER_CLIENT -gt 0 ]; then

            $IPTABLES -A OUTPUT -o $EXTERNAL_INTERFACE -p TCP \
        --sport $UNPRIVPORTS --dport 79 -s $EXTERNAL_IP -d $ANYWHERE -j ACCEPT

            $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -p TCP -m state --state ESTABLISHED,RELATED \
        --sport 79 --dport $UNPRIVPORTS -s $ANYWHERE -d $EXTERNAL_IP -j ACCEPT

            if [ $MASQUERADING -gt 0 ]; then
               $IPTABLES -A FORWARD -p TCP -s $INTERNAL_NETWORK --sport $UNPRIVPORTS --dport 79 -j ACCEPT
               $IPTABLES -A FORWARD -m state --state ESTABLISHED,RELATED -p TCP -d $INTERNAL_NETWORK \
                --sport 79 --dport $UNPRIVPORTS -j ACCEPT
            fi

            if [ $VERBOSE -gt 0 ]; then
                echo "firewall: Clients may access remote FINGER servers"
            fi

        fi

        #
        # FINGER (79) - Allowing remote client access to a local finger server (dangerous!)
        #

        if [ $FINGER_SERVER -gt 0 ]; then

            $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -p TCP \
        --sport $UNPRIVPORTS --dport 79 -s $MY_FINGER_CLIENTS -d $EXTERNAL_IP -j ACCEPT

            $IPTABLES -A OUTPUT -o $EXTERNAL_INTERFACE -p TCP -m state --state ESTABLISHED,RELATED \
        --sport 79 --dport $UNPRIVPORTS -s $EXTERNAL_IP -d $MY_FINGER_CLIENTS -j ACCEPT

            if [ $VERBOSE -gt 0 ]; then
                echo "firewall: Remote clients may access local FINGER server"
            fi

        fi

        #
        # WHOIS (43) - Accessing a remote WHOIS server as a client
        #

        if [ $WHOIS_CLIENT -gt 0 ]; then

            $IPTABLES -A OUTPUT -o $EXTERNAL_INTERFACE -p TCP \
        --sport $UNPRIVPORTS --dport 43 -s $EXTERNAL_IP -d $ANYWHERE -j ACCEPT

            $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -p TCP -m state --state ESTABLISHED,RELATED \
        --sport 43 --dport $UNPRIVPORTS -s $ANYWHERE -d $EXTERNAL_IP -j ACCEPT

            if [ $MASQUERADING -gt 0 ]; then
               $IPTABLES -A FORWARD -p TCP -s $INTERNAL_NETWORK --sport $UNPRIVPORTS --dport 43 -j ACCEPT
               $IPTABLES -A FORWARD -m state --state ESTABLISHED,RELATED -p TCP -d $INTERNAL_NETWORK \
                --sport 43 --dport $UNPRIVPORTS -j ACCEPT
            fi

            if [ $VERBOSE -gt 0 ]; then
                echo "firewall: Clients may access remote WHOIS servers"
            fi

        fi

        #
        # GOPHER (70) - Accessing a remote GOPHER server as a client
        #

        if [ $GOPHER_CLIENT -gt 0 ]; then

            $IPTABLES -A OUTPUT -o $EXTERNAL_INTERFACE -p TCP \
        --sport $UNPRIVPORTS --dport 70 -s $EXTERNAL_IP -d $ANYWHERE -j ACCEPT

            $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -p TCP -m state --state ESTABLISHED,RELATED \
        --sport 70 --dport $UNPRIVPORTS -s $ANYWHERE -d $EXTERNAL_IP -j ACCEPT

            if [ $MASQUERADING -gt 0 ]; then
               $IPTABLES -A FORWARD -p TCP -s $INTERNAL_NETWORK --sport $UNPRIVPORTS --dport 70 -j ACCEPT
               $IPTABLES -A FORWARD -m state --state ESTABLISHED,RELATED -p TCP -d $INTERNAL_NETWORK \
                --sport 70 --dport $UNPRIVPORTS -j ACCEPT
            fi

            if [ $VERBOSE -gt 0 ]; then
                echo "firewall: Clients may access remote GOPHER servers"
            fi

        fi

        #
        # WAIS (210) - Accessing a remote WAIS server as a client
        #

        if [ $WAIS_CLIENT -gt 0 ]; then

            $IPTABLES -A OUTPUT -o $EXTERNAL_INTERFACE -p TCP \
        --sport $UNPRIVPORTS --dport 210 -s $EXTERNAL_IP -d $ANYWHERE -j ACCEPT

            $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -p TCP -m state --state ESTABLISHED,RELATED \
        --sport 210 --dport $UNPRIVPORTS -s $ANYWHERE -d $EXTERNAL_IP -j ACCEPT

            if [ $MASQUERADING -gt 0 ]; then
               $IPTABLES -A FORWARD -p TCP -s $INTERNAL_NETWORK --sport $UNPRIVPORTS --dport 210 -j ACCEPT
               $IPTABLES -A FORWARD -m state --state ESTABLISHED,RELATED -p TCP -d $INTERNAL_NETWORK \
                --sport 210 --dport $UNPRIVPORTS -j ACCEPT
            fi

            if [ $VERBOSE -gt 0 ]; then
                echo "firewall: Clients may access remote WAIS servers"
            fi

        fi

        #
        # Real Video (554) - Real Video Client
        #

        if [ $RV_CLIENT -gt 0 ]; then
            $IPTABLES -A OUTPUT -o $EXTERNAL_INTERFACE -p TCP \
               --sport $UNPRIVPORTS --dport 554 -s $EXTERNAL_IP -d $ANYWHERE -j ACCEPT

            $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -p TCP \
               --sport 554 -s $ANYWHERE -d $EXTERNAL_IP -j ACCEPT

            if [ $MASQUERADING -gt 0 ]; then
               $IPTABLES -A FORWARD -p TCP -s $INTERNAL_NETWORK --sport $UNPRIVPORTS --dport 554 -j ACCEPT
               $IPTABLES -A FORWARD -m state --state ESTABLISHED,RELATED -p TCP -d $INTERNAL_NETWORK \
                --sport 554 --dport $UNPRIVPORTS -j ACCEPT
            fi

            if [ $VERBOSE -gt 0 ]; then
                echo "firewall: Real Video client enabled"
            fi
        fi

        #
        # PPTP (1723) - Accessing PPTP servers as a client
        #

        if [ $PPTP_CLIENT -gt 0 ]; then

            $IPTABLES -A OUTPUT -o $EXTERNAL_INTERFACE -p TCP \
                --sport $UNPRIVPORTS --dport 1723 \
                -s $EXTERNAL_IP -d $ANYWHERE -j ACCEPT

            $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -p TCP  \
                --sport 1723 --dport $UNPRIVPORTS \
                -s $ANYWHERE -d $EXTERNAL_IP \
                -m state --state ESTABLISHED,RELATED -j ACCEPT

            $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -p 47 -j ACCEPT
            $IPTABLES -A OUTPUT -o $EXTERNAL_INTERFACE -p 47 -j ACCEPT

            if [ $MASQUERADING -gt 0 ]; then
               $IPTABLES -A INPUT -i $INTERNAL_INTERFACE -p 47 -j ACCEPT
               $IPTABLES -A OUTPUT -o $INTERNAL_INTERFACE -p 47 -j ACCEPT

               $IPTABLES -A FORWARD -p TCP -s $INTERNAL_NETWORK \
                  --sport $UNPRIVPORTS --dport 1723 -j ACCEPT

               $IPTABLES -A FORWARD -p TCP -d $INTERNAL_NETWORK \
                  -m state --state ESTABLISHED,RELATED \
                  --sport 1723 --dport $UNPRIVPORTS -j ACCEPT

               $IPTABLES -A FORWARD -p 47 -s $INTERNAL_NETWORK -j ACCEPT
               $IPTABLES -A FORWARD -p 47 -d $INTERNAL_NETWORK -j ACCEPT

            fi

            if [ $VERBOSE -gt 0 ]; then
                echo "firewall: Clients may access remote PPTP servers"
            fi

        fi



        #
        # UDP - Accept only on selected ports
        #

        #
        # TRACEROUTE
        #
        # Traceroute usually uses -s 32769:65535 -d 33434:33523
        #

        if [ $OUTBOUND_TRACEROUTE -gt 0 ]; then

            # Enable outgoing TRACEROUTE requests

            $IPTABLES -A OUTPUT -o $EXTERNAL_INTERFACE -p UDP \
        --sport $TRACEROUTE_SRC_PORTS --dport $TRACEROUTE_DEST_PORTS \
        -s $EXTERNAL_IP -d $ANYWHERE -j ACCEPT

            if [ $MASQUERADING -gt 0 ]; then
               $IPTABLES -A FORWARD -p UDP -s $INTERNAL_NETWORK --sport $TRACEROUTE_SRC_PORTS \
                --dport $TRACEROUTE_DEST_PORTS -j ACCEPT
               $IPTABLES -A FORWARD -p UDP -d $INTERNAL_NETWORK --sport $TRACEROUTE_DEST_PORTS \
                --dport $TRACEROUTE_SRC_PORTS -j ACCEPT
            fi

            if [ $VERBOSE -gt 0 ]; then
                echo "firewall: Outbound TRACEROUTE enabled"
            fi

        fi

        if [ $INBOUND_TRACEROUTE -gt 0 ]; then

            # Enable incoming TRACEROUTE query

            $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -p UDP \
        --sport $TRACEROUTE_SRC_PORTS --dport $TRACEROUTE_DEST_PORTS \
        -s $ANYWHERE -d $EXTERNAL_IP -j ACCEPT

            if [ $MASQUERADING -gt 0 ]; then
               $IPTABLES -A FORWARD -p UDP -d $INTERNAL_NETWORK --sport $TRACEROUTE_SRC_PORTS \
                --dport $TRACEROUTE_DEST_PORTS -j ACCEPT
            fi

            if [ $VERBOSE -gt 0 ]; then
                echo "firewall: Inbound TRACEROUTE enabled"
            fi

        fi


        #
        # DHCP Server
        #
        # This assumes that you're running a DHCP server on your firewall to
        # supply IP addresses to your internal network using dhcpd.  See any
        # of several DHCP HowTo sites for the actual server setup.
        #

        if [ $DHCP_SERVER -gt 0 ]; then

            $IPTABLES -A INPUT -i $INTERNAL_INTERFACE -p udp -s $BROADCAST_SRC \
        -d $BROADCAST_DEST --sport 67:68 --dport 67:68 -j ACCEPT
            $IPTABLES -A OUTPUT -o $INTERNAL_INTERFACE -p udp -s $INTERNAL_IP \
        --sport 67:68 --dport 67:68 -j ACCEPT
            $IPTABLES -A FORWARD -p udp -s $INTERNAL_NETWORK --sport 67:68 --dport 67:68 -j ACCEPT
            $IPTABLES -A FORWARD -p udp -d $INTERNAL_NETWORK --sport 67:68 --dport 67:68 -j ACCEPT

           if [ $VERBOSE -gt 0 ]; then
                echo "firewall: DHCP Server enabled"
           fi

        fi

        #
        # NTP (123) - Accessing remote Network Time Servers
        #

        if [ $NTP_CLIENT -gt 0 ]; then

            $IPTABLES -A OUTPUT -o $EXTERNAL_INTERFACE -p UDP \
        --sport $UNPRIVPORTS --dport 123 -s $EXTERNAL_IP -d $ANYWHERE -j ACCEPT

            $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -p UDP \
        --sport 123 --dport $UNPRIVPORTS -s $ANYWHERE -d $EXTERNAL_IP -j ACCEPT

            $IPTABLES -A OUTPUT -o $EXTERNAL_INTERFACE -p UDP \
        --sport 123 --dport 123 -s $EXTERNAL_IP -d $ANYWHERE -j ACCEPT

            $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -p UDP \
        --sport 123 --dport 123 -s $ANYWHERE -d $EXTERNAL_IP -j ACCEPT

            if [ $MASQUERADING -gt 0 ]; then
               $IPTABLES -A FORWARD -p UDP -s $INTERNAL_NETWORK --sport $UNPRIVPORTS --dport 123 -j ACCEPT
               $IPTABLES -A FORWARD -p UDP -d $INTERNAL_NETWORK --sport 123 --dport $UNPRIVPORTS -j ACCEPT
               $IPTABLES -A FORWARD -p UDP -s $INTERNAL_NETWORK --sport 123 --dport 123 -j ACCEPT
               $IPTABLES -A FORWARD -p UDP -d $INTERNAL_NETWORK --sport 123 --dport 123 -j ACCEPT
            fi

            if [ $VERBOSE -gt 0 ]; then
                echo "firewall: NTP Client enabled"
            fi

        fi


        #
        # ICQ (4000) - The Miribilis ICQ Client
        #

        if [ $ICQ_CLIENT -gt 0 ]; then

            $IPTABLES -A OUTPUT -o $EXTERNAL_INTERFACE -p TCP \
        --sport $UNPRIVPORTS --dport 5190 -s $EXTERNAL_IP -d $ANYWHERE -j ACCEPT

            $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -p TCP \
        --sport 5190 --dport $UNPRIVPORTS -s $ANYWHERE -d $EXTERNAL_IP -j ACCEPT

            if [ $MASQUERADING -gt 0 ]; then
               $IPTABLES -A FORWARD -p TCP -s $INTERNAL_NETWORK --sport $UNPRIVPORTS --dport 5190 -j ACCEPT
               $IPTABLES -A FORWARD -p TCP -d $INTERNAL_NETWORK --sport 5190 --dport $UNPRIVPORTS -j ACCEPT
            fi

            if [ $VERBOSE -gt 0 ]; then
                echo "firewall: ICQ Client enabled"
            fi

        fi

        #
        # ICQ (4000) - The Miribilis ICQ Client
        #

        if [ $RADIO_CLIENT -gt 0 ]; then

            $IPTABLES -A OUTPUT -o $EXTERNAL_INTERFACE -p TCP \
        --sport $UNPRIVPORTS --dport 1755 -s $EXTERNAL_IP -d $ANYWHERE -j ACCEPT

            $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -p TCP \
        --sport 1755 --dport $UNPRIVPORTS -s $ANYWHERE -d $EXTERNAL_IP -j ACCEPT

            if [ $MASQUERADING -gt 0 ]; then
               $IPTABLES -A FORWARD -p TCP -s $INTERNAL_NETWORK --sport $UNPRIVPORTS --dport 1755 -j ACCEPT
               $IPTABLES -A FORWARD -p TCP -d $INTERNAL_NETWORK --sport 1755 --dport $UNPRIVPORTS -j ACCEPT
            fi

            if [ $VERBOSE -gt 0 ]; then
                echo "firewall: RADIO Client enabled"
            fi

        fi

        if [ $IRC_CLIENT -gt 0 ]; then

                $IPTABLES -A OUTPUT -o $EXTERNAL_INTERFACE -p TCP \
                --sport $UNPRIVPORTS --dport 6667 -s $EXTERNAL_IP -d $ANYWHERE -j ACCEPT

                $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -p TCP \
                --sport 6667 --dport $UNPRIVPORTS -s $ANYWHERE -d $EXTERNAL_IP -j ACCEPT

                if [ $MASQUERADING -gt 0 ]; then
                        $IPTABLES -A FORWARD -p TCP -s $INTERNAL_NETWORK --sport $UNPRIVPORTS --dport 6667 -j ACCEPT
                        $IPTABLES -A FORWARD -p TCP -d $INTERNAL_NETWORK --sport 6667 --dport $UNPRIVPORTS -j ACCEPT
                fi

                if [ $VERBOSE -gt 0 ]; then
                        echo "firewall: IRC Client enabled"
                fi

        fi



        #
        # GAMES
        # Half-Life/CounterStrike
        #

        if [ $HALF_LIFE -gt 0 ]; then

            $IPTABLES -A OUTPUT -o $EXTERNAL_INTERFACE -p UDP \
        --sport 27000:27050 --dport $UNPRIVPORTS -s $EXTERNAL_IP -d $ANYWHERE -j ACCEPT

            $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -p UDP \
        -m state --state RELATED,ESTABLISHED,NEW --dport 27000:27050 -s $ANYWHERE \
        -d $EXTERNAL_IP -j ACCEPT

            if [ $MASQUERADING -gt 0 ]; then
               $IPTABLES -A FORWARD -p UDP -s $INTERNAL_NETWORK --sport 27000:27050 --dport $UNPRIVPORTS -j ACCEPT
               $IPTABLES -A FORWARD -p UDP -d $INTERNAL_NETWORK -m state --state RELATED,ESTABLISHED,NEW --dport 27000:27050 -j ACCEPT
            fi

            if [ $VERBOSE -gt 0 ]; then
                echo "firewall: Half-Life/CounterStrike game ports enabled"
            fi

        fi

        #
        # Return to Castle Wolfenstein
        #

        if [ $WOLF_CLIENT -gt 0 ]; then

            $IPTABLES  -A OUTPUT -o $EXTERNAL_INTERFACE -p UDP \
        --sport $UNPRIVPORTS --dport 27950:27965 -s $EXTERNAL_IP -d $ANYWHERE -j ACCEPT
            $IPTABLES  -A INPUT -i $EXTERNAL_INTERFACE -p UDP \
        --sport 27950:27965 --dport $UNPRIVPORTS -s $ANYWHERE -d $EXTERNAL_IP -j ACCEPT

            if [ $MASQUERADING -gt 0 ]; then
               $IPTABLES -A FORWARD -p UDP -s $INTERNAL_NETWORK --sport $UNPRIVPORTS --dport 27950:27965 -j ACCEPT
               $IPTABLES -A FORWARD -p UDP -d $INTERNAL_NETWORK --sport 27950:27965 --dport $UNPRIVPORTS -j ACCEPT
            fi

            if [ $VERBOSE -gt 0 ]; then
                echo "firewall: Castle Wolfenstein game ports enabled"
            fi
        fi


        # -------------------------------------------------------------

        #
        # Spoofing and Bad Addresses
        #

        # (These would usually be logged as well, rather than just DROPped, but the
        # huge number of attempts from China to play game(s) generates huge log files;
        # if you want them logged, put the rules in firewall.local)

        # Refuse spoofed packets.
        # Ignore blatantly illegal source addresses.
        # Protect yourself from sending to bad addresses.

        # Refuse spoofed packets pretending to be from
        # the external interface's IP address.

        $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -s $EXTERNAL_IP -j LnD

        # Refuse packets claiming to be to or from a Class-A private network.

        $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -s $CLASS_A -j LnD
        $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -d $CLASS_A -j LnD
        $IPTABLES -A OUTPUT -o $EXTERNAL_INTERFACE -s $CLASS_A -j LnD
        $IPTABLES -A OUTPUT -o $EXTERNAL_INTERFACE -d $CLASS_A -j LnD

        # Refuse packets claiming to be to or from a Class-B private network.

        $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -s $CLASS_B -j LnD
        $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -d $CLASS_B -j LnD
        $IPTABLES -A OUTPUT -o $EXTERNAL_INTERFACE -s $CLASS_B -j LnD
        $IPTABLES -A OUTPUT -o $EXTERNAL_INTERFACE -d $CLASS_B -j LnD

        # Refuse packets claiming to be to or from a Class-C private network.

        $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -s $CLASS_C -j LnD
        $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -d $CLASS_C -j LnD
        $IPTABLES -A OUTPUT -o $EXTERNAL_INTERFACE -s $CLASS_C -j LnD
        $IPTABLES -A OUTPUT -o $EXTERNAL_INTERFACE -d $CLASS_C -j LnD

        # Refuse packets claiming to be from the loopback.

        $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -s $LOOPBACK_NETWORK -j DROP
        $IPTABLES -A OUTPUT -o $EXTERNAL_INTERFACE -s $LOOPBACK_NETWORK -j DROP

        # Refuse malformed broadcast packets.

        $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -s $BROADCAST_DEST -j LnD
        $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -d $BROADCAST_SRC  -j LnD
        $IPTABLES -A OUTPUT -o $EXTERNAL_INTERFACE -s $BROADCAST_DEST -j LnD
        $IPTABLES -A OUTPUT -o $EXTERNAL_INTERFACE -d $BROADCAST_SRC  -j LnD

        # Refuse Class-D Multicast addresses.
        # Multicast is only illegal as a source address.
        # Multicast uses UDP.

        $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -s $CLASS_D_MULTICAST -j LnD
        $IPTABLES -A OUTPUT -o $EXTERNAL_INTERFACE -s $CLASS_D_MULTICAST -j LnR

        # Refuse Class-E reserved IP addresses.

        $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -s $CLASS_E_RESERVED_NET -j LnD
        $IPTABLES -A OUTPUT -o $EXTERNAL_INTERFACE -d $CLASS_E_RESERVED_NET -j LnR


        # -------------------------------------------------------------

        #
        # DROP (on input), REJECT (output) and LOG anything else on the external (red) interface
        #

        $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -p TCP \
            -s $ANYWHERE -j LnD

        $IPTABLES -A OUTPUT -o $EXTERNAL_INTERFACE -p TCP \
            -s $ANYWHERE -j LnR

        $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -p UDP \
            -s $ANYWHERE -j LnD

        $IPTABLES -A OUTPUT -o $EXTERNAL_INTERFACE -p UDP \
            -s $ANYWHERE -j LnR

        $IPTABLES -A INPUT -i $EXTERNAL_INTERFACE -p ICMP \
            -s $ANYWHERE -j LnD

        $IPTABLES -A OUTPUT -o $EXTERNAL_INTERFACE -p ICMP \
            -s $ANYWHERE -j LnR

        # -------------------------------------------------------------

        # Zero counts
        $IPTABLES -Z

        # -------------------------------------------------------------


	/etc/firewall/addons/wow.sh
	/etc/firewall/addons/shoutcast.sh
	/etc/firewall/addons/adslfree.sh

        echo "done"
        touch /var/lock/subsys/firewall
        echo
        ;;

  status)
        if [ -f /var/lock/subsys/firewall ]; then
            echo "Firewall started and configured"
        else
            echo "Firewall stopped"
        fi
        exit 0
        ;;

  restart|reload)
        $0 stop $2
        $0 start $2
        ;;

  stop)
        echo "Shutting down Firewall services"

        # Turn off IP Forwarding
        echo 0 >/proc/sys/net/ipv4/ip_forward

        # Turn off dynamic IP hacking
        echo 0 > /proc/sys/net/ipv4/ip_dynaddr

        # Flush the rule chains
        $IPTABLES -F

        # Delete custom chains
        $IPTABLES -X

        # Zero counts
        $IPTABLES -Z

        # Set the default policy to DROP
        $IPTABLES -P INPUT DROP
        $IPTABLES -P OUTPUT DROP
        $IPTABLES -P FORWARD DROP

        # Allow unlimited traffic on the loopback interface
        $IPTABLES -A INPUT -i $LOOPBACK_INTERFACE -j ACCEPT
        $IPTABLES -A OUTPUT -o $LOOPBACK_INTERFACE -j ACCEPT

        # Open the configuration file
        if [ -f $FW_CONF ]; then
            . $FW_CONF

            # Allow unlimited traffic on the loopback interface
            $IPTABLES -A INPUT -i $LOOPBACK_INTERFACE -j ACCEPT
            $IPTABLES -A OUTPUT -o $LOOPBACK_INTERFACE -j ACCEPT

            if [ $MASQUERADING -gt 0 ]; then
                # Allow unlimited local traffic on the internal interface
                $IPTABLES -A INPUT -i $INTERNAL_INTERFACE -j ACCEPT
                $IPTABLES -A OUTPUT -o $INTERNAL_INTERFACE -j ACCEPT
            fi
        else
            echo "firewall: No configuration file found at /etc/firewall/firewall.conf"
            exit 1
        fi

        rm -f /var/lock/subsys/firewall
        echo
        ;;
  *)
        echo "Usage: /etc/rc.d/init.d/firewall.$IPTABLES {start|stop|status|restart|reload}"
        exit 1
esac

exit 0
